@@ -8,7 +8,7 @@ except ImportError:
 
 from impacket import ImpactPacket
 from impacket import ImpactDecoder
-from impacket.ImpactPacket import TCPOption
+from impacket.ImpactPacket import TCPOption, array_tobytes
 from impacket.examples import logger
 from impacket.examples import os_ident
 
@@ -354,10 +354,7 @@ class UDPCommandResponder(OpenUDPResponder):
           #in_onion[O_UDP].get_uh_dport() == self.port)
 
    def buildAnswer(self, in_onion):
-       try:
-           cmd = in_onion[O_UDP_DATA].get_bytes().tobytes()
-       except AttributeError:  # Python < 3.2
-           cmd = in_onion[O_UDP_DATA].get_bytes().tostring()
+       cmd = array_tobytes(in_onion[O_UDP_DATA].get_bytes())
        if cmd[:4] == 'cmd:': cmd = cmd[4:].strip()
        print("Got command: %r" % cmd)
 
