@@ -29,6 +29,7 @@ from struct import unpack
 from impacket import version
 from impacket.dcerpc.v5 import transport, rrp, scmr, rpcrt
 from impacket.examples import logger
+from impacket.examples.utils import parse_target
 from impacket.system_errors import ERROR_NO_MORE_ITEMS
 from impacket.structure import hexdump
 from impacket.smbconnection import SMBConnection
@@ -401,15 +402,7 @@ if __name__ == '__main__':
     else:
         logging.getLogger().setLevel(logging.INFO)
 
-    import re
-
-    domain, username, password, remoteName = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
-        options.target).groups('')
-
-    # In case the password contains '@'
-    if '@' in remoteName:
-        password = password + '@' + remoteName.rpartition('@')[0]
-        remoteName = remoteName.rpartition('@')[2]
+    domain, username, password, remoteName = parse_target(options.target)
 
     if options.target_ip is None:
         options.target_ip = remoteName
