@@ -50,14 +50,15 @@ import argparse
 import logging
 
 from impacket.examples import logger
+from impacket.examples.utils import parse_target
 from impacket import version
-from impacket.dcerpc.v5.dcomrt import DCOMConnection
+from impacket.dcerpc.v5.dcomrt import DCOMConnection, COMVERSION
 from impacket.dcerpc.v5.dcom import wmi
 from impacket.dcerpc.v5.dtypes import NULL
 
 
 class WMIPERSISTENCE:
-    def __init__(self, username = '', password = '', domain = '', options= None):
+    def __init__(self, username='', password='', domain='', options=None):
         self.__username = username
         self.__password = password
         self.__domain = domain
@@ -80,7 +81,7 @@ class WMIPERSISTENCE:
 
         iInterface = dcom.CoCreateInstanceEx(wmi.CLSID_WbemLevel1Login,wmi.IID_IWbemLevel1Login)
         iWbemLevel1Login = wmi.IWbemLevel1Login(iInterface)
-        iWbemServices= iWbemLevel1Login.NTLMLogin('//./root/subscription', NULL, NULL)
+        iWbemServices = iWbemLevel1Login.NTLMLogin('//./root/subscription', NULL, NULL)
         iWbemLevel1Login.RemRelease()
 
         if self.__options.action.upper() == 'REMOVE':
@@ -100,8 +101,8 @@ class WMIPERSISTENCE:
                                 r'Filter="__EventFilter.Name=\"EF_%s\""' % (
                                 self.__options.name, self.__options.name)))
         else:
-            activeScript ,_ = iWbemServices.GetObject('ActiveScriptEventConsumer')
-            activeScript =  activeScript.SpawnInstance()
+            activeScript, _ = iWbemServices.GetObject('ActiveScriptEventConsumer')
+            activeScript = activeScript.SpawnInstance()
             activeScript.Name = self.__options.name
             activeScript.ScriptingEngine = 'VBScript'
             activeScript.CreatorSID = [1, 2, 0, 0, 0, 0, 0, 5, 32, 0, 0, 0, 32, 2, 0, 0]
@@ -110,14 +111,14 @@ class WMIPERSISTENCE:
                 iWbemServices.PutInstance(activeScript.marshalMe()))
         
             if options.filter is not None:
-                eventFilter,_ = iWbemServices.GetObject('__EventFilter')
-                eventFilter =  eventFilter.SpawnInstance()
+                eventFilter, _ = iWbemServices.GetObject('__EventFilter')
+                eventFilter = eventFilter.SpawnInstance()
                 eventFilter.Name = 'EF_%s' % self.__options.name
-                eventFilter.CreatorSID =  [1, 2, 0, 0, 0, 0, 0, 5, 32, 0, 0, 0, 32, 2, 0, 0]
+                eventFilter.CreatorSID = [1, 2, 0, 0, 0, 0, 0, 5, 32, 0, 0, 0, 32, 2, 0, 0]
                 eventFilter.Query = options.filter
                 eventFilter.QueryLanguage = 'WQL'
                 eventFilter.EventNamespace = r'root\cimv2'
-                self.checkError('Adding EventFilter EF_%s'% self.__options.name, 
+                self.checkError('Adding EventFilter EF_%s' % self.__options.name,
                     iWbemServices.PutInstance(eventFilter.marshalMe()))
 
             else:
@@ -136,20 +137,24 @@ class WMIPERSISTENCE:
                 eventFilter.Query = 'select * from __TimerEvent where TimerID = "TI_%s" ' % self.__options.name
                 eventFilter.QueryLanguage = 'WQL'
                 eventFilter.EventNamespace = r'root\subscription'
-                self.checkError('Adding EventFilter EF_%s'% self.__options.name, 
+                self.checkError('Adding EventFilter EF_%s' % self.__options.name,
                     iWbemServices.PutInstance(eventFilter.marshalMe()))
 
-            filterBinding,_ = iWbemServices.GetObject('__FilterToConsumerBinding')
-            filterBinding =  filterBinding.SpawnInstance()
+            filterBinding, _ = iWbemServices.GetObject('__FilterToConsumerBinding')
+            filterBinding = filterBinding.SpawnInstance()
             filterBinding.Filter = '__EventFilter.Name="EF_%s"' % self.__options.name
             filterBinding.Consumer = 'ActiveScriptEventConsumer.Name="%s"' % self.__options.name
             filterBinding.CreatorSID = [1, 2, 0, 0, 0, 0, 0, 5, 32, 0, 0, 0, 32, 2, 0, 0]
+            # Even when the default value of DeliveryQoS is 0, we're explicitly assigning it to
+            # avoid the default tag
+            filterBinding.DeliveryQoS = 0  # WMIMSG_FLAG_QOS_SYNCHRONOUS
 
             self.checkError('Adding FilterToConsumerBinding',
                 iWbemServices.PutInstance(filterBinding.marshalMe()))
 
         dcom.disconnect()
 
+
 # Process command-line arguments.
 if __name__ == '__main__':
     # Init the example's logger theme
@@ -161,6 +166,8 @@ if __name__ == '__main__':
 
     parser.add_argument('target', action='store', help='[domain/][username[:password]@]<address>')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
+    parser.add_argument('-com-version', action='store', metavar = "MAJOR_VERSION:MINOR_VERSION", help='DCOM version, '
+                        'format is MAJOR_VERSION:MINOR_VERSION e.g. 5.7')
     subparsers = parser.add_subparsers(help='actions', dest='action')
 
     # A start command
@@ -195,7 +202,6 @@ if __name__ == '__main__':
 
     options = parser.parse_args()
 
-        
     if options.debug is True:
         logging.getLogger().setLevel(logging.DEBUG)
         # Print the Library's installation path
@@ -203,21 +209,20 @@ if __name__ == '__main__':
     else:
         logging.getLogger().setLevel(logging.INFO)
 
+    if options.com_version is not None:
+        try:
+            major_version, minor_version = options.com_version.split('.')
+            COMVERSION.set_default_version(int(major_version), int(minor_version))
+        except Exception:
+            logging.error("Wrong COMVERSION format, use dot separated integers e.g. \"5.7\"")
+            sys.exit(1)
 
     if options.action.upper() == 'INSTALL':
         if (options.filter is None and options.timer is None) or  (options.filter is not None and options.timer is not None):
             logging.error("You have to either specify -filter or -timer (and not both)")
             sys.exit(1)
 
-    import re
-
-    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
-        options.target).groups('')
-
-    #In case the password contains '@'
-    if '@' in address:
-        password = password + '@' + address.rpartition('@')[0]
-        address = address.rpartition('@')[2]
+    domain, username, password, address = parse_target(options.target)
 
     try:
         if domain is None:
