@@ -22,6 +22,16 @@ import sys
 from binascii import hexlify
 from functools import reduce
 
+# Alias function for compatibility with both Python <3.2 `tostring` and `fromstring` methods, and
+# Python >=3.2 `tobytes` and `tostring`
+if sys.version_info[0] >= 3 and sys.version_info[1] >= 2:
+    array_tobytes = lambda array_object: array_object.tobytes()
+    array_frombytes = lambda array_object, bytes: array_object.frombytes(bytes)
+else:
+    array_tobytes = lambda array_object: array_object.tostring()
+    array_frombytes = lambda array_object, bytes: array_object.fromstring(bytes)
+
+
 """Classes to build network packets programmatically.
 
 Each protocol layer is represented by an object, and these objects are
@@ -59,11 +69,8 @@ class PacketBuffer(object):
         self.__bytes = array.array('B', data)
 
     def get_buffer_as_string(self):
-        "Returns the packet buffer as a bytes object"
-        try:
-            return self.__bytes.tobytes()
-        except AttributeError:  # Python < 3.2
-            return self.__bytes.tostring()
+        "Returns the packet buffer as a string object"
+        return array_tobytes(self.__bytes)
 
     def get_bytes(self):
         "Returns the packet buffer as an array"
@@ -100,10 +107,7 @@ class PacketBuffer(object):
             bytes = self.__bytes[index:]
         else:
             bytes = self.__bytes[index:index+2]
-        try:
-            (value,) = struct.unpack(order + 'H', bytes.tobytes())
-        except AttributeError:  # Python < 3.2
-            (value,) = struct.unpack(order + 'H', bytes.tostring())
+        (value,) = struct.unpack(order + 'H', array_tobytes(bytes))
         return value
 
     def set_long(self, index, value, order = '!'):
@@ -122,10 +126,7 @@ class PacketBuffer(object):
             bytes = self.__bytes[index:]
         else:
             bytes = self.__bytes[index:index+4]
-        try:
-            (value,) = struct.unpack(order + 'L', bytes.tobytes())
-        except AttributeError:  # Python < 3.2
-            (value,) = struct.unpack(order + 'L', bytes.tostring())
+        (value,) = struct.unpack(order + 'L', array_tobytes(bytes))
         return value
 
     def set_long_long(self, index, value, order = '!'):
@@ -144,10 +145,7 @@ class PacketBuffer(object):
             bytes = self.__bytes[index:]
         else:
             bytes = self.__bytes[index:index+8]
-        try:
-            (value,) = struct.unpack(order + 'Q', bytes.tobytes())
-        except AttributeError:  # Python < 3.2
-            (value,) = struct.unpack(order + 'Q', bytes.tostring())
+        (value,) = struct.unpack(order + 'Q', array_tobytes(bytes))
         return value
 
 
@@ -158,10 +156,7 @@ class PacketBuffer(object):
             bytes = self.__bytes[index:]
         else:
             bytes = self.__bytes[index:index+4]
-        try:
-            return socket.inet_ntoa(bytes.tobytes())
-        except AttributeError:  # Python < 3.2
-            return socket.inet_ntoa(bytes.tostring())
+        return socket.inet_ntoa(array_tobytes(bytes))
 
     def set_ip_address(self, index, ip_string):
         "Set 4-byte value at 'index' from 'ip_string'"
@@ -211,10 +206,7 @@ class PacketBuffer(object):
 
         diff = index + size - curlen
         if diff > 0:
-            try:
-                self.__bytes.frombytes(b'\0' * diff)
-            except AttributeError:  # Python < 3.2
-                self.__bytes.fromstring(b'\0' * diff)
+            array_frombytes(self.__bytes, b'\0' * diff)
             if orig_index < 0:
                 orig_index -= diff
 
@@ -737,10 +729,7 @@ class LinuxSLL(Header):
 
     def get_addr(self):
         "Returns the sender's address field"
-        try:
-            return self.get_bytes()[6:14].tobytes()
-        except AttributeError:  # Python < 3.2
-            return self.get_bytes()[6:14].tostring()
+        return array_tobytes(self.get_bytes()[6:14])
 
     def set_ether_type(self, aValue):
         "Set ethernet data type field to 'aValue'"
@@ -818,10 +807,7 @@ class IP(Header):
         # Pad to a multiple of 4 bytes
         num_pad = (4 - (len(my_bytes) % 4)) % 4
         if num_pad:
-            try:
-                my_bytes.frombytes(b"\0"* num_pad)
-            except AttributeError:  # Python < 3.2
-                my_bytes.fromstring(b"\0"* num_pad)
+            array_frombytes(my_bytes, b"\0" * num_pad)
 
         # only change ip_hl value if options are present
         if len(self.__option_list):
@@ -832,10 +818,10 @@ class IP(Header):
         if self.auto_checksum:
             self.set_ip_sum(self.compute_checksum(my_bytes))
 
-        try:
-            return my_bytes.tobytes() + (child_data or b"")
-        except AttributeError:  # Python < 3.2
-            return my_bytes.tostring() + (child_data or b"")
+        if child_data is None:
+            return array_tobytes(my_bytes)
+        else:
+            return array_tobytes(my_bytes) + child_data
 
 
 
@@ -859,10 +845,7 @@ class IP(Header):
 
         size_str = struct.pack("!H", tmp_size)
 
-        try:
-            pseudo_buf.frombytes(size_str)
-        except AttributeError:  # Python < 3.2
-            pseudo_buf.fromstring(size_str)
+        array_frombytes(pseudo_buf, size_str)
         return pseudo_buf
 
     def add_option(self, option):
@@ -1322,11 +1305,8 @@ class UDP(Header):
 
             buffer += self.get_bytes()
             data = self.get_data_as_string()
-            if data:
-                try:
-                    buffer.frombytes(data)
-                except AttributeError:  # Python < 3.2
-                    buffer.fromstring(data)
+            if(data):
+                array_frombytes(buffer, data)
             self.set_uh_sum(self.compute_checksum(buffer))
 
     def get_header_size(self):
@@ -1515,11 +1495,8 @@ class TCP(Header):
         buffer += self.get_padded_options()
 
         data = self.get_data_as_string()
-        if data:
-            try:
-                buffer.frombytes(data)
-            except AttributeError:  # Python < 3.2
-                buffer.fromstring(data)
+        if(data):
+            array_frombytes(buffer, data)
 
         res = self.compute_checksum(buffer)
 
@@ -1537,10 +1514,10 @@ class TCP(Header):
         bytes = self.get_bytes() + self.get_padded_options()
         data = self.get_data_as_string()
 
-        try:
-            return bytes.tobytes() + (data or b"")
-        except AttributeError:  # Python < 3.2
-            return bytes.tostring() + (data or b"")
+        if data:
+            return array_tobytes(bytes) + data
+        else:
+            return array_tobytes(bytes)
 
     def load_header(self, aBuffer):
         self.set_bytes_from_string(aBuffer[:20])
@@ -1595,10 +1572,7 @@ class TCP(Header):
             op_buf += op.get_bytes()
         num_pad = (4 - (len(op_buf) % 4)) % 4
         if num_pad:
-            try:
-                op_buf.frombytes(b"\0" * num_pad)
-            except AttributeError:  # Python < 3.2
-                op_buf.fromstring(b"\0" * num_pad)
+            array_frombytes(op_buf, "\0" * num_pad)
         return op_buf
 
     def __str__(self):
