@@ -25,7 +25,8 @@
 # [X] Implement a ping mechanism, otherwise the garbage collector at the server shuts down the objects if 
 #    not used, returning RPC_E_DISCONNECTED
 #
-
+from __future__ import division
+from __future__ import print_function
 import socket
 from struct import pack
 from threading import Timer, currentThread
@@ -70,7 +71,7 @@ class DCERPCSessionError(DCERPCException):
         DCERPCException.__init__(self, error_string, error_code, packet)
 
     def __str__( self ):
-        if hresult_errors.ERROR_MESSAGES.has_key(self.error_code):
+        if self.error_code in hresult_errors.ERROR_MESSAGES:
             error_msg_short = hresult_errors.ERROR_MESSAGES[self.error_code][0]
             error_msg_verbose = hresult_errors.ERROR_MESSAGES[self.error_code][1]
             return 'DCOM SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
@@ -153,21 +154,37 @@ class handle_t(NDRSTRUCT):
         ('context_handle_attributes',ULONG),
         ('context_handle_uuid',UUID),
     )
-    def __init__(self, data = None,isNDR64 = False):
+
+    def __init__(self, data=None, isNDR64=False):
         NDRSTRUCT.__init__(self, data, isNDR64)
-        self['context_handle_uuid'] = '\x00'*20
+        self['context_handle_uuid'] = b'\x00'*16
+
+    def isNull(self):
+        return self['context_handle_uuid'] == b'\x00'*16
 
 # 2.2.11 COMVERSION
 class COMVERSION(NDRSTRUCT):
+    default_major_version = 5
+    default_minor_version = 7
+
     structure = (
         ('MajorVersion',USHORT),
         ('MinorVersion',USHORT),
     )
+
+    @classmethod
+    def set_default_version(cls, major_version=None, minor_version=None):
+        # Set default dcom version for all new COMVERSION objects.
+        if major_version is not None:
+            cls.default_major_version = major_version
+        if minor_version is not None:
+            cls.default_minor_version = minor_version
+
     def __init__(self, data = None,isNDR64 = False):
         NDRSTRUCT.__init__(self, data, isNDR64)
         if data is None:
-            self['MajorVersion'] = 5
-            self['MinorVersion'] = 6
+            self['MajorVersion'] = self.default_major_version
+            self['MinorVersion'] = self.default_minor_version
 
 class PCOMVERSION(NDRPOINTER):
     referent = (
@@ -328,7 +345,7 @@ class DUALSTRINGARRAYPACKED(NDRSTRUCT):
         ('wSecurityOffset',USHORT),
         ('aStringArray',':'),
     )
-    def getDataLen(self, data):
+    def getDataLen(self, data, offset=0):
         return self['wNumEntries']*2
 
 # 2.2.18.7 OBJREF_EXTENDED
@@ -963,20 +980,20 @@ class DCOMConnection:
 
     @classmethod
     def addOid(cls, target, oid):
-        if DCOMConnection.OID_ADD.has_key(target) is False:
+        if (target in DCOMConnection.OID_ADD) is False:
             DCOMConnection.OID_ADD[target] = set()
         DCOMConnection.OID_ADD[target].add(oid)
-        if DCOMConnection.OID_SET.has_key(target) is False:
+        if (target in DCOMConnection.OID_SET) is False:
             DCOMConnection.OID_SET[target] = {}
             DCOMConnection.OID_SET[target]['oids'] = set()
             DCOMConnection.OID_SET[target]['setid'] = 0
 
     @classmethod
     def delOid(cls, target, oid):
-        if DCOMConnection.OID_DEL.has_key(target) is False:
+        if (target in DCOMConnection.OID_DEL) is False:
             DCOMConnection.OID_DEL[target] = set()
         DCOMConnection.OID_DEL[target].add(oid)
-        if DCOMConnection.OID_SET.has_key(target) is False:
+        if (target in DCOMConnection.OID_SET) is False:
             DCOMConnection.OID_SET[target] = {}
             DCOMConnection.OID_SET[target]['oids'] = set()
             DCOMConnection.OID_SET[target]['setid'] = 0
@@ -991,18 +1008,18 @@ class DCOMConnection:
             for target in DCOMConnection.OID_SET:
                 addedOids = set()
                 deletedOids = set()
-                if DCOMConnection.OID_ADD.has_key(target):
+                if target in DCOMConnection.OID_ADD:
                     addedOids = DCOMConnection.OID_ADD[target]
                     del(DCOMConnection.OID_ADD[target])
 
-                if DCOMConnection.OID_DEL.has_key(target):
+                if target in DCOMConnection.OID_DEL:
                     deletedOids = DCOMConnection.OID_DEL[target]
                     del(DCOMConnection.OID_DEL[target])
 
                 objExporter = IObjectExporter(DCOMConnection.PORTMAPS[target])
 
                 if len(addedOids) > 0 or len(deletedOids) > 0:
-                    if DCOMConnection.OID_SET[target].has_key('setid'):
+                    if 'setid' in DCOMConnection.OID_SET[target]:
                         setId = DCOMConnection.OID_SET[target]['setid']
                     else:
                         setId = 0
@@ -1012,7 +1029,7 @@ class DCOMConnection:
                     DCOMConnection.OID_SET[target]['setid'] = resp['pSetId']
                 else:
                     objExporter.SimplePing(DCOMConnection.OID_SET[target]['setid'])
-        except Exception, e:
+        except Exception as e:
             # There might be exceptions when sending packets 
             # We should try to continue tho.
             LOG.error(str(e))
@@ -1021,7 +1038,7 @@ class DCOMConnection:
         DCOMConnection.PINGTIMER = Timer(120,DCOMConnection.pingServer)
         try:
             DCOMConnection.PINGTIMER.start()
-        except Exception, e:
+        except Exception as e:
             if str(e).find('threads can only be started once') < 0:
                 raise e
 
@@ -1031,7 +1048,7 @@ class DCOMConnection:
                 DCOMConnection.PINGTIMER = Timer(120, DCOMConnection.pingServer)
             try:
                 DCOMConnection.PINGTIMER.start()
-            except Exception, e:
+            except Exception as e:
                 if str(e).find('threads can only be started once') < 0:
                     raise e
 
@@ -1069,7 +1086,7 @@ class DCOMConnection:
                 DCOMConnection.PINGTIMER.cancel()
                 DCOMConnection.PINGTIMER.join()
                 DCOMConnection.PINGTIMER = None
-        if INTERFACE.CONNECTIONS.has_key(self.__target):
+        if self.__target in INTERFACE.CONNECTIONS:
             del(INTERFACE.CONNECTIONS[self.__target][currentThread().getName()])
         self.__portmap.disconnect()
         #print INTERFACE.CONNECTIONS
@@ -1124,7 +1141,7 @@ class INTERFACE:
             self.__objRef = objRef
             self.__ipidRemUnknown = ipidRemUnknown
             # We gotta check if we have a container inside our connection list, if not, create
-            if INTERFACE.CONNECTIONS.has_key(self.__target) is not True:
+            if (self.__target in INTERFACE.CONNECTIONS) is not True:
                 INTERFACE.CONNECTIONS[self.__target] = {}
                 INTERFACE.CONNECTIONS[self.__target][currentThread().getName()] = {}
 
@@ -1213,9 +1230,9 @@ class INTERFACE:
 
 
     def connect(self, iid = None):
-        if INTERFACE.CONNECTIONS.has_key(self.__target) is True:
-            if INTERFACE.CONNECTIONS[self.__target].has_key(currentThread().getName()) and \
-                            INTERFACE.CONNECTIONS[self.__target][currentThread().getName()].has_key(self.__oxid) is True:
+        if (self.__target in INTERFACE.CONNECTIONS) is True:
+            if currentThread().getName() in INTERFACE.CONNECTIONS[self.__target] and \
+                            (self.__oxid in INTERFACE.CONNECTIONS[self.__target][currentThread().getName()]) is True:
                 dce = INTERFACE.CONNECTIONS[self.__target][currentThread().getName()][self.__oxid]['dce']
                 currentBinding = INTERFACE.CONNECTIONS[self.__target][currentThread().getName()][self.__oxid]['currentBinding']
                 if currentBinding == iid:
@@ -1308,7 +1325,7 @@ class INTERFACE:
         dce = self.get_dce_rpc()
         try:
             resp = dce.request(req, uuid)
-        except Exception, e:
+        except Exception as e:
             if str(e).find('RPC_E_DISCONNECTED') >= 0:
                 msg = str(e) + '\n'
                 msg += "DCOM keep-alive pinging it might not be working as expected. You can't be idle for more than 14 minutes!\n"
@@ -1397,13 +1414,13 @@ class IObjectExporter:
         for protSeq in arRequestedProtseqs:
             request['arRequestedProtseqs'].append(protSeq)
         resp = self.__portmap.request(request)
-        Oxids = ''.join(pack('<H', x) for x in resp['ppdsaOxidBindings']['aStringArray'])
+        Oxids = b''.join(pack('<H', x) for x in resp['ppdsaOxidBindings']['aStringArray'])
         strBindings = Oxids[:resp['ppdsaOxidBindings']['wSecurityOffset']*2]
 
         done = False
         stringBindings = list()
         while not done:
-            if strBindings[0] == '\x00' and strBindings[1] == '\x00':
+            if strBindings[0:1] == b'\x00' and strBindings[1:2] == b'\x00':
                 done = True
             else:
                 binding = STRINGBINDING(strBindings)
@@ -1467,13 +1484,13 @@ class IObjectExporter:
         for protSeq in arRequestedProtseqs:
             request['arRequestedProtseqs'].append(protSeq)
         resp = self.__portmap.request(request)
-        Oxids = ''.join(pack('<H', x) for x in resp['ppdsaOxidBindings']['aStringArray'])
+        Oxids = b''.join(pack('<H', x) for x in resp['ppdsaOxidBindings']['aStringArray'])
         strBindings = Oxids[:resp['ppdsaOxidBindings']['wSecurityOffset']*2]
 
         done = False
         stringBindings = list()
         while not done:
-            if strBindings[0] == '\x00' and strBindings[1] == '\x00':
+            if strBindings[0:1] == b'\x00' and strBindings[1:2] == b'\x00':
                 done = True
             else:
                 binding = STRINGBINDING(strBindings)
@@ -1489,13 +1506,13 @@ class IObjectExporter:
         request = ServerAlive2()
         resp = self.__portmap.request(request)
 
-        Oxids = ''.join(pack('<H', x) for x in resp['ppdsaOrBindings']['aStringArray'])
+        Oxids = b''.join(pack('<H', x) for x in resp['ppdsaOrBindings']['aStringArray'])
         strBindings = Oxids[:resp['ppdsaOrBindings']['wSecurityOffset']*2]
 
         done = False
         stringBindings = list()
         while not done:
-            if strBindings[0] == '\x00' and strBindings[1] == '\x00':
+            if strBindings[0:1] == b'\x00' and strBindings[1:2] == b'\x00':
                 done = True
             else:
                 binding = STRINGBINDING(strBindings)
@@ -1539,14 +1556,14 @@ class IActivation:
 
         ipidRemUnknown = resp['pipidRemUnknown']
 
-        Oxids = ''.join(pack('<H', x) for x in resp['ppdsaOxidBindings']['aStringArray'])
+        Oxids = b''.join(pack('<H', x) for x in resp['ppdsaOxidBindings']['aStringArray'])
         strBindings = Oxids[:resp['ppdsaOxidBindings']['wSecurityOffset']*2]
         securityBindings = Oxids[resp['ppdsaOxidBindings']['wSecurityOffset']*2:]
 
         done = False
         stringBindings = list()
         while not done:
-            if strBindings[0] == '\x00' and strBindings[1] == '\x00':
+            if strBindings[0:1] == b'\x00' and strBindings[1:2] == b'\x00':
                 done = True
             else:
                 binding = STRINGBINDING(strBindings)
@@ -1557,14 +1574,14 @@ class IActivation:
         while not done:
             if len(securityBindings) < 2:
                 done = True
-            elif securityBindings[0] == '\x00' and securityBindings[1] == '\x00':
+            elif securityBindings[0:1] == b'\x00' and securityBindings[1:2 ]== b'\x00':
                 done = True
             else:
                 secBinding = SECURITYBINDING(securityBindings)
                 securityBindings = securityBindings[len(secBinding):]
 
         classInstance = CLASS_INSTANCE(ORPCthis, stringBindings)
-        return IRemUnknown2(INTERFACE(classInstance, ''.join(resp['ppInterfaceData'][0]['abData']), ipidRemUnknown,
+        return IRemUnknown2(INTERFACE(classInstance, b''.join(resp['ppInterfaceData'][0]['abData']), ipidRemUnknown,
                                       target=self.__portmap.get_rpc_transport().getRemoteHost()))
 
 
@@ -1599,7 +1616,7 @@ class IRemoteSCMActivator:
         clsid['Data'] = CLSID_ScmRequestInfo
         activationBLOB['CustomHeader']['pclsid'].append(clsid)
 
-        properties = ''
+        properties = b''
         # InstantiationInfo
         instantiationInfo = InstantiationInfoData()
         instantiationInfo['classId'] = clsId
@@ -1617,7 +1634,7 @@ class IRemoteSCMActivator:
         activationBLOB['CustomHeader']['pSizes'].append(dword)
         instantiationInfo['thisSize'] = dword['Data']
 
-        properties += marshaled + '\xFA'*pad
+        properties += marshaled + b'\xFA'*pad
 
         # ActivationContextInfoData
         activationInfo = ActivationContextInfoData()
@@ -1630,7 +1647,7 @@ class IRemoteSCMActivator:
         dword['Data'] = len(marshaled) + pad
         activationBLOB['CustomHeader']['pSizes'].append(dword)
 
-        properties += marshaled + '\xFA'*pad
+        properties += marshaled + b'\xFA'*pad
 
         # ServerLocation
         locationInfo = LocationInfoData()
@@ -1655,7 +1672,7 @@ class IRemoteSCMActivator:
         dword['Data'] = len(marshaled) + pad
         activationBLOB['CustomHeader']['pSizes'].append(dword)
 
-        properties += marshaled + '\xFA'*pad
+        properties += marshaled + b'\xFA'*pad
 
         activationBLOB['Property'] = properties
 
@@ -1667,21 +1684,21 @@ class IRemoteSCMActivator:
         objrefcustom['pObjectData'] = activationBLOB.getData()
         objrefcustom['ObjectReferenceSize'] = len(objrefcustom['pObjectData'])+8
 
-        request['pActProperties']['ulCntData'] = len(str(objrefcustom))
-        request['pActProperties']['abData'] = list(str(objrefcustom))
+        request['pActProperties']['ulCntData'] = len(objrefcustom.getData())
+        request['pActProperties']['abData'] = list(objrefcustom.getData())
         resp = self.__portmap.request(request)
         # Now let's parse the answer and build an Interface instance
 
-        objRefType = OBJREF(''.join(resp['ppActProperties']['abData']))['flags']
+        objRefType = OBJREF(b''.join(resp['ppActProperties']['abData']))['flags']
         objRef = None
         if objRefType == FLAGS_OBJREF_CUSTOM:
-            objRef = OBJREF_CUSTOM(''.join(resp['ppActProperties']['abData']))
+            objRef = OBJREF_CUSTOM(b''.join(resp['ppActProperties']['abData']))
         elif objRefType == FLAGS_OBJREF_HANDLER:
-            objRef = OBJREF_HANDLER(''.join(resp['ppActProperties']['abData']))
+            objRef = OBJREF_HANDLER(b''.join(resp['ppActProperties']['abData']))
         elif objRefType == FLAGS_OBJREF_STANDARD:
-            objRef = OBJREF_STANDARD(''.join(resp['ppActProperties']['abData']))
+            objRef = OBJREF_STANDARD(b''.join(resp['ppActProperties']['abData']))
         elif objRefType == FLAGS_OBJREF_EXTENDED:
-            objRef = OBJREF_EXTENDED(''.join(resp['ppActProperties']['abData']))
+            objRef = OBJREF_EXTENDED(b''.join(resp['ppActProperties']['abData']))
         else:
             LOG.error("Unknown OBJREF Type! 0x%x" % objRefType)
 
@@ -1690,24 +1707,22 @@ class IRemoteSCMActivator:
 
         propOutput = activationBlob['Property'][:activationBlob['CustomHeader']['pSizes'][0]['Data']]
         scmReply = activationBlob['Property'][
-                   activationBlob['CustomHeader']['pSizes'][0]['Data']:activationBlob['CustomHeader']['pSizes'][0][
-                                                                           'Data'] +
-                                                                       activationBlob['CustomHeader']['pSizes'][1][
-                                                                           'Data']]
+                   activationBlob['CustomHeader']['pSizes'][0]['Data']:activationBlob['CustomHeader']['pSizes'][0]['Data'] +
+                                                                       activationBlob['CustomHeader']['pSizes'][1]['Data']]
 
         scmr = ScmReplyInfoData()
         size = scmr.fromString(scmReply)
         # Processing the scmReply
         scmr.fromStringReferents(scmReply[size:])
         ipidRemUnknown = scmr['remoteReply']['ipidRemUnknown']
-        Oxids = ''.join(pack('<H', x) for x in scmr['remoteReply']['pdsaOxidBindings']['aStringArray'])
+        Oxids = b''.join(pack('<H', x) for x in scmr['remoteReply']['pdsaOxidBindings']['aStringArray'])
         strBindings = Oxids[:scmr['remoteReply']['pdsaOxidBindings']['wSecurityOffset']*2]
         securityBindings = Oxids[scmr['remoteReply']['pdsaOxidBindings']['wSecurityOffset']*2:]
 
         done = False
         stringBindings = list()
         while not done:
-            if strBindings[0] == '\x00' and strBindings[1] == '\x00':
+            if strBindings[0:1] == b'\x00' and strBindings[1:2] == b'\x00':
                 done = True
             else:
                 binding = STRINGBINDING(strBindings)
@@ -1718,7 +1733,7 @@ class IRemoteSCMActivator:
         while not done:
             if len(securityBindings) < 2:
                 done = True
-            elif securityBindings[0] == '\x00' and securityBindings[1] == '\x00':
+            elif securityBindings[0:1] == b'\x00' and securityBindings[1:2] == b'\x00':
                 done = True
             else:
                 secBinding = SECURITYBINDING(securityBindings)
@@ -1732,7 +1747,7 @@ class IRemoteSCMActivator:
         classInstance = CLASS_INSTANCE(ORPCthis, stringBindings)
         classInstance.set_auth_level(scmr['remoteReply']['authnHint'])
         classInstance.set_auth_type(self.__portmap.get_auth_type())
-        return IRemUnknown2(INTERFACE(classInstance, ''.join(propsOut['ppIntfData'][0]['abData']), ipidRemUnknown,
+        return IRemUnknown2(INTERFACE(classInstance, b''.join(propsOut['ppIntfData'][0]['abData']), ipidRemUnknown,
                                       target=self.__portmap.get_rpc_transport().getRemoteHost()))
 
     def RemoteCreateInstance(self, clsId, iid):
@@ -1764,7 +1779,7 @@ class IRemoteSCMActivator:
         clsid['Data'] = CLSID_ScmRequestInfo
         activationBLOB['CustomHeader']['pclsid'].append(clsid)
 
-        properties = ''
+        properties = b''
         # InstantiationInfo
         instantiationInfo = InstantiationInfoData()
         instantiationInfo['classId'] = clsId
@@ -1782,7 +1797,7 @@ class IRemoteSCMActivator:
         activationBLOB['CustomHeader']['pSizes'].append(dword)
         instantiationInfo['thisSize'] = dword['Data']
 
-        properties += marshaled + '\xFA'*pad
+        properties += marshaled + b'\xFA'*pad
 
         # ActivationContextInfoData
         activationInfo = ActivationContextInfoData()
@@ -1795,7 +1810,7 @@ class IRemoteSCMActivator:
         dword['Data'] = len(marshaled) + pad
         activationBLOB['CustomHeader']['pSizes'].append(dword)
 
-        properties += marshaled + '\xFA'*pad
+        properties += marshaled + b'\xFA'*pad
 
         # ServerLocation
         locationInfo = LocationInfoData()
@@ -1820,7 +1835,7 @@ class IRemoteSCMActivator:
         dword['Data'] = len(marshaled) + pad
         activationBLOB['CustomHeader']['pSizes'].append(dword)
 
-        properties += marshaled + '\xFA'*pad
+        properties += marshaled + b'\xFA'*pad
 
         activationBLOB['Property'] = properties
 
@@ -1832,22 +1847,22 @@ class IRemoteSCMActivator:
         objrefcustom['pObjectData'] = activationBLOB.getData()
         objrefcustom['ObjectReferenceSize'] = len(objrefcustom['pObjectData'])+8
 
-        request['pActProperties']['ulCntData'] = len(str(objrefcustom))
-        request['pActProperties']['abData'] = list(str(objrefcustom))
+        request['pActProperties']['ulCntData'] = len(objrefcustom.getData())
+        request['pActProperties']['abData'] = list(objrefcustom.getData())
         resp = self.__portmap.request(request)
 
         # Now let's parse the answer and build an Interface instance
 
-        objRefType = OBJREF(''.join(resp['ppActProperties']['abData']))['flags']
+        objRefType = OBJREF(b''.join(resp['ppActProperties']['abData']))['flags']
         objRef = None
         if objRefType == FLAGS_OBJREF_CUSTOM:
-            objRef = OBJREF_CUSTOM(''.join(resp['ppActProperties']['abData']))
+            objRef = OBJREF_CUSTOM(b''.join(resp['ppActProperties']['abData']))
         elif objRefType == FLAGS_OBJREF_HANDLER:
-            objRef = OBJREF_HANDLER(''.join(resp['ppActProperties']['abData']))
+            objRef = OBJREF_HANDLER(b''.join(resp['ppActProperties']['abData']))
         elif objRefType == FLAGS_OBJREF_STANDARD:
-            objRef = OBJREF_STANDARD(''.join(resp['ppActProperties']['abData']))
+            objRef = OBJREF_STANDARD(b''.join(resp['ppActProperties']['abData']))
         elif objRefType == FLAGS_OBJREF_EXTENDED:
-            objRef = OBJREF_EXTENDED(''.join(resp['ppActProperties']['abData']))
+            objRef = OBJREF_EXTENDED(b''.join(resp['ppActProperties']['abData']))
         else:
             LOG.error("Unknown OBJREF Type! 0x%x" % objRefType)
 
@@ -1856,24 +1871,22 @@ class IRemoteSCMActivator:
 
         propOutput = activationBlob['Property'][:activationBlob['CustomHeader']['pSizes'][0]['Data']]
         scmReply = activationBlob['Property'][
-                   activationBlob['CustomHeader']['pSizes'][0]['Data']:activationBlob['CustomHeader']['pSizes'][0][
-                                                                           'Data'] +
-                                                                       activationBlob['CustomHeader']['pSizes'][1][
-                                                                           'Data']]
+                   activationBlob['CustomHeader']['pSizes'][0]['Data']:activationBlob['CustomHeader']['pSizes'][0]['Data'] +
+                                                                       activationBlob['CustomHeader']['pSizes'][1]['Data']]
 
         scmr = ScmReplyInfoData()
         size = scmr.fromString(scmReply)
         # Processing the scmReply
         scmr.fromStringReferents(scmReply[size:])
         ipidRemUnknown = scmr['remoteReply']['ipidRemUnknown']
-        Oxids = ''.join(pack('<H', x) for x in scmr['remoteReply']['pdsaOxidBindings']['aStringArray'])
+        Oxids = b''.join(pack('<H', x) for x in scmr['remoteReply']['pdsaOxidBindings']['aStringArray'])
         strBindings = Oxids[:scmr['remoteReply']['pdsaOxidBindings']['wSecurityOffset']*2]
         securityBindings = Oxids[scmr['remoteReply']['pdsaOxidBindings']['wSecurityOffset']*2:]
 
         done = False
         stringBindings = list()
         while not done:
-            if strBindings[0] == '\x00' and strBindings[1] == '\x00':
+            if strBindings[0:1] == b'\x00' and strBindings[1:2] == b'\x00':
                 done = True
             else:
                 binding = STRINGBINDING(strBindings)
@@ -1884,7 +1897,7 @@ class IRemoteSCMActivator:
         while not done:
             if len(securityBindings) < 2:
                 done = True
-            elif securityBindings[0] == '\x00' and securityBindings[1] == '\x00':
+            elif securityBindings[0:1] == b'\x00' and securityBindings[1:2] == b'\x00':
                 done = True
             else:
                 secBinding = SECURITYBINDING(securityBindings)
@@ -1898,5 +1911,5 @@ class IRemoteSCMActivator:
         classInstance = CLASS_INSTANCE(ORPCthis, stringBindings)
         classInstance.set_auth_level(scmr['remoteReply']['authnHint'])
         classInstance.set_auth_type(self.__portmap.get_auth_type())
-        return IRemUnknown2(INTERFACE(classInstance, ''.join(propsOut['ppIntfData'][0]['abData']), ipidRemUnknown,
+        return IRemUnknown2(INTERFACE(classInstance, b''.join(propsOut['ppIntfData'][0]['abData']), ipidRemUnknown,
                                       target=self.__portmap.get_rpc_transport().getRemoteHost()))
