@@ -33,7 +33,7 @@ class DCERPCSessionError(DCERPCException):
 
     def __str__(self):
         key = self.error_code
-        if system_errors.ERROR_MESSAGES.has_key(key):
+        if key in system_errors.ERROR_MESSAGES:
             error_msg_short = system_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = system_errors.ERROR_MESSAGES[key][1]
             return 'EVEN6 SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
@@ -315,7 +315,7 @@ def hEvtRpcQueryNext(dce, handle, numRequestedRecords, timeOutEnd=1000):
     while status == system_errors.ERROR_MORE_DATA:
         try:
             resp = dce.request(request)
-        except DCERPCException, e:
+        except DCERPCException as e:
             if str(e).find('ERROR_NO_MORE_ITEMS') < 0:
                 raise
             elif str(e).find('ERROR_TIMEOUT') < 0:
@@ -342,4 +342,3 @@ def hEvtRpcGetChannelList(dce):
     request['Flags'] = 0
     resp = dce.request(request)
     return resp
-
