@@ -35,7 +35,7 @@ class DCERPCSessionError(DCERPCException):
 
     def __str__( self ):
         key = self.error_code
-        if system_errors.ERROR_MESSAGES.has_key(key):
+        if key in system_errors.ERROR_MESSAGES:
             error_msg_short = system_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = system_errors.ERROR_MESSAGES[key][1] 
             return 'WKST SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
@@ -127,10 +127,10 @@ class LPWKSSVC_IMPERSONATE_HANDLE(NDRPOINTER):
 # 2.2.3.1 NETSETUP_JOIN_STATUS
 class NETSETUP_JOIN_STATUS(NDRENUM):
     class enumItems(Enum):
-        NetSetupUnknownStatus = 1
-        NetSetupUnjoined      = 2
-        NetSetupWorkgroupName = 3
-        NetSetupDomainName    = 4
+        NetSetupUnknownStatus = 0
+        NetSetupUnjoined      = 1
+        NetSetupWorkgroupName = 2
+        NetSetupDomainName    = 3
 
 # 2.2.3.2 NETSETUP_NAME_TYPE
 class NETSETUP_NAME_TYPE(NDRENUM):
@@ -467,11 +467,11 @@ class WKSTA_TRANSPORT_ENUM_STRUCT(NDRSTRUCT):
 
 # 2.2.5.17 JOINPR_USER_PASSWORD
 class WCHAR_ARRAY(WIDESTR):
-    def getDataLen(self, data):
+    def getDataLen(self, data, offset=0):
         return JOIN_MAX_PASSWORD_LENGTH
 
 class CHAR_ARRAY(NDRUniFixedArray):
-    def getDataLen(self, data):
+    def getDataLen(self, data, offset=0):
         return JOIN_OBFUSCATOR_LENGTH
 
 class JOINPR_USER_PASSWORD(NDRSTRUCT):
@@ -483,7 +483,7 @@ class JOINPR_USER_PASSWORD(NDRSTRUCT):
 # 2.2.5.18 JOINPR_ENCRYPTED_USER_PASSWORD
 class JOINPR_ENCRYPTED_USER_PASSWORD(NDRSTRUCT):
     structure = (
-        ('Buffer', '524s=""'),
+        ('Buffer', '524s=b""'),
     )
     def getAlignment(self):
         return 1
@@ -1180,4 +1180,3 @@ def hNetrUseDel(dce, useName, forceLevel=USE_LOTS_OF_FORCE):
     request['UseName'] = checkNullString(useName)
     request['ForceLevel'] = forceLevel
     return dce.request(request)
-
