@@ -31,16 +31,17 @@ import struct
 
 from pyasn1.codec.der import decoder
 
-import asn1
-import constants
+from . import asn1
+from . import constants
 
 
-class KerberosException(Exception): pass
+class KerberosException(Exception):
+    pass
 
 def _asn1_decode(data, asn1Spec):
-    if isinstance(data, basestring):
+    if isinstance(data, str) or isinstance(data,bytes):
         data, substrate = decoder.decode(data, asn1Spec=asn1Spec)
-        if substrate != '':
+        if substrate != b'':
             raise KerberosException("asn1 encoding invalid")
     return data
 
@@ -62,11 +63,18 @@ If the value contains no realm, then default_realm will be used."""
         if value is None:
             return
 
+        try:               # Python 2
+            if isinstance(value, unicode):
+                value = value.encode('utf-8')
+        except NameError:  # Python 3
+            if isinstance(value, bytes):
+                value = value.decode('utf-8')
+
         if isinstance(value, Principal):
             self.type = value.type
             self.components = value.components[:]
             self.realm = value.realm
-        elif isinstance(value, basestring):
+        elif isinstance(value, str):
             m = re.match(r'((?:[^\\]|\\.)+?)(@((?:[^\\@]|\\.)+))?$', value)
             if not m:
                 raise KerberosException("invalid principal syntax")
@@ -85,7 +93,7 @@ If the value contains no realm, then default_realm will be used."""
         elif len(value) == 2:
             self.components = value[0]
             self.realm = value[-1]
-            if isinstance(self.components, basestring):
+            if isinstance(self.components, str):
                 self.components = [self.components]
         elif len(value) >= 2:
             self.components = value[0:-1]
@@ -97,15 +105,13 @@ If the value contains no realm, then default_realm will be used."""
             self.type = type
 
     def __eq__(self, other):
-        if isinstance(other, basestring):
-            other = Principal(other)
+        if isinstance (other, str):
+            other = Principal (other)
 
-        return (self.type == constants.PrincipalNameType.NT_UNKNOWN.value or \
-                other.type == constants.PrincipalNameType.NT_UNKNOWN.value or \
-                self.type == other.type) and \
-            all(map(lambda a,b: a == b,
-                    self.components, other.components)) and \
-            self.realm == other.realm
+        return (self.type == constants.PrincipalNameType.NT_UNKNOWN.value or
+                other.type == constants.PrincipalNameType.NT_UNKNOWN.value or
+                self.type == other.type) and all (map (lambda a, b: a == b, self.components, other.components)) and \
+               self.realm == other.realm
 
     def __str__(self):
         def quote_component(comp):
@@ -190,7 +196,7 @@ class EncryptedData(object):
         if (kvno is None) or (kvno.hasValue() is False):
             self.kvno = False
         else:
-            self.kvno = True
+            self.kvno = kvno
         self.ciphertext = str(data.getComponentByName('cipher'))
         return self
 
@@ -254,15 +260,15 @@ class KerberosTime(object):
 
 if __name__ == '__main__':
     # TODO marc: turn this into a real test
-    print Principal("marc")
-    print Principal(("marc", None))
-    print Principal((("marc",), None))
-    print Principal("marc@ATHENA.MIT.EDU")
-    print Principal("marc", default_realm="ATHENA.MIT.EDU")
-    print Principal("marc@ATHENA.MIT.EDU", default_realm="EXAMPLE.COM")
-    print Principal(("marc", "ATHENA.MIT.EDU"))
-    print Principal((("marc"), "ATHENA.MIT.EDU"))
-    print Principal("marc/root")
-    print Principal(("marc", "root", "ATHENA.MIT.EDU"))
-    print Principal((("marc", "root"), "ATHENA.MIT.EDU"))
-    print Principal("marc\\/root")
+    print(Principal("marc"))
+    print(Principal(("marc", None)))
+    print(Principal((("marc",), None)))
+    print(Principal("marc@ATHENA.MIT.EDU"))
+    print(Principal("marc", default_realm="ATHENA.MIT.EDU"))
+    print(Principal("marc@ATHENA.MIT.EDU", default_realm="EXAMPLE.COM"))
+    print(Principal(("marc", "ATHENA.MIT.EDU")))
+    print(Principal((("marc"), "ATHENA.MIT.EDU")))
+    print(Principal("marc/root"))
+    print(Principal(("marc", "root", "ATHENA.MIT.EDU")))
+    print(Principal((("marc", "root"), "ATHENA.MIT.EDU")))
+    print(Principal("marc\\/root"))
